-- -------
-- B-BUME
-- -------

-- Zustzlichen Index fr die Tabelle "Gemeinden" anlegen:
CREATE INDEX Gemeinden_name_ix ON Gemeinden(name);

-- Index wieder lschen:
DROP INDEX Gemeinden_name_ix;


-- ---------
-- QUADTREES
-- ---------

-- Fixed Indexing

-- neuen Index angelegen:
CREATE INDEX GeoDbLand_geo_ix
ON GeoDbLand(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX
PARAMETERS('SDO_LEVEL=6');

-- Index mit genderten Parametern neu aufbauen:
ALTER INDEX GeoDbLand_geo_ix REBUILD
PARAMETERS('SDO_LEVEL=4');

-- Hybrid Indexing

-- Bisherigen Index lschen:
DROP INDEX GeoDbLand_geo_ix;

-- Neuen Index anlegen:
CREATE INDEX GeoDbLand_geo_ix
ON GeoDbLand(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX
PARAMETERS('SDO_LEVEL=4 SDO_NUMTILES=10');

-- oder einfach abndern:
ALTER INDEX GeoDbLand_geo_ix REBUILD
PARAMETERS('SDO_LEVEL=4 SDO_NUMTILES=10');

-- Index-Informationen abfragen

SELECT INDEX_NAME, TABLE_NAME, COLUMN_NAME, SDO_INDEX_TYPE, SDO_INDEX_TABLE
FROM USER_SDO_INDEX_INFO;

SELECT SDO_INDEX_NAME, SDO_INDEX_TABLE, SDO_INDEX_DIMS, SDO_LEVEL, SDO_NUMTILES, SDO_MAXLEVEL
FROM USER_SDO_INDEX_METADATA;

-- Indextabelle abfragen

-- Fix Indexing mit grober Zerlegung:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=4');

-- Abfrage der Anzahl der z-Werte pro Geometrie (Namen der Indextabelle anpassen!):
SELECT l.name, COUNT(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid
GROUP BY l.name;

-- Fix Indexing mit feinerer Zerlegung:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=6');

-- Abfrage der Anzahl der z-Werte pro Geometrie (Namen der Indextabelle anpassen!):
SELECT l.name, COUNT(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid
GROUP BY l.name;

-- Hybrid Indexing:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=4 SDO_NUMTILES=10');

-- Abfrage der Anzahl der z-Werte pro Geometrie (Namen der Indextabelle anpassen!):
SELECT l.name, COUNT(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid
GROUP BY l.name;

-- Betrachtung der Gruppierung beim Haus (Namen der Indextabelle anpassen!):
SELECT l.name,i.sdo_groupcode, COUNT(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid 
WHERE l.name = 'Haus'
GROUP BY l.name,i.sdo_groupcode;

-- Fix Indexing mit feinerer Zerlegung:
ALTER INDEX GeoDbLand_geo_ix REBUILD PARAMETERS('SDO_LEVEL=6');
-- Betrachtung der Zellenstati beim Haus (Namen der Indextabelle anpassen!):
SELECT l.name, i.sdo_status, COUNT(i.sdo_code)
FROM MDQT_774C$ i INNER JOIN GeoDbLand l
ON i.sdo_rowid = l.rowid 
WHERE l.name = 'Haus'
GROUP BY l.name, i.sdo_status;


-- -------
-- R-BUME
-- -------

-- Bisherigen Index lschen:
DROP INDEX GeoDbLand_geo_ix;
-- Neuen Index anlegen:
CREATE INDEX GeoDbLand_geo_ix
ON GeoDbLand(geo)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- oder einfach abndern:
ALTER INDEX GeoDbLand_geo_ix REBUILD;

-- Index fr geografische Koordinaten

-- Indexe lschen und anlegen:
DROP INDEX Gemeinden_centrum_ix;

CREATE INDEX Gemeinden_centrum_ix
ON Gemeinden(centrum)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

DROP INDEX Ortsnetze_gebiet_ix;

CREATE INDEX Ortsnetze_gebiet_ix
ON Ortsnetze(gebiet)
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

-- Restriktion auf Punkte:
ALTER INDEX Gemeinden_centrum_ix REBUILD
PARAMETERS('LAYER_GTYPE=POINT');

-- Versuch ein Polygon zu speichern:
UPDATE Gemeinden SET centrum=
SDO_GEOMETRY (2003, 8307, NULL, SDO_ELEM_INFO_ARRAY(1,1003,1),
SDO_ORDINATE_ARRAY(
 8.214289,53.087776, 8.305773,53.148083, 8.173382,53.184449, 8.214289,53.087776 ))
WHERE gkz = 3403000;

-- Index-Informationen abfragen

SELECT INDEX_NAME, TABLE_NAME, COLUMN_NAME, SDO_INDEX_TYPE, SDO_INDEX_TABLE
FROM USER_SDO_INDEX_INFO;

SELECT SDO_INDEX_TABLE, SDO_RTREE_HEIGHT, SDO_INDEX_NAME, SDO_RTREE_NUM_NODES,            
       SDO_RTREE_FANOUT, SDO_RTREE_ROOT, SDO_RTREE_PCTFREE, SDO_LAYER_GTYPE              
FROM USER_SDO_INDEX_METADATA;

-- Verteilung auf zwei Tabellen

-- Index ndern:
ALTER INDEX Ortsnetze_gebiet_ix REBUILD
PARAMETERS('SDO_NON_LEAF_TBL=TRUE');

-- Tabellennamen anzeigen:
SELECT t.table_name
FROM USER_TABLES t INNER JOIN USER_SDO_INDEX_INFO i
ON SUBSTR(t.table_name,5,5) = SUBSTR(i.sdo_index_table,5,5)
WHERE i.index_name = 'ORTSNETZE_GEBIET_IX';

-- Index auf Funktionen

-- vordefinierte Funktion
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('ORTSNETZE','MDSYS.SDO_GEOM.SDO_CENTROID(GEBIET,0.001)', SDO_DIM_ARRAY(
  SDO_DIM_ELEMENT('X',-180,180,1),
  SDO_DIM_ELEMENT('Y',-90,90,1)
) , 4326);
COMMIT;

CREATE INDEX Ortsnetze_gebiet_centroid_ix
ON Ortsnetze (MDSYS.SDO_GEOM.SDO_CENTROID(gebiet,0.001))
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

SELECT * FROM Ortsnetze
WHERE SDO_FILTER (SDO_GEOM.SDO_CENTROID(gebiet,0.001), SDO_GEOMETRY('POLYGON((8 53,8 54,9 54,9 53,8 53))',4326)) = 'TRUE';

-- benutzerdefinierte Funktion
CREATE OR REPLACE FUNCTION schwerpunkt (geo SDO_GEOMETRY) RETURN SDO_GEOMETRY DETERMINISTIC
IS
BEGIN
  RETURN SDO_GEOM.SDO_CENTROID(geo,0.001);
END;
/

-- in nachfolgender Anweisung ist zunchst der Schemaname einzusetzen
INSERT INTO USER_SDO_GEOM_METADATA
VALUES ('ORTSNETZE','Schemaname.SCHWERPUNKT(GEBIET)', SDO_DIM_ARRAY(
  SDO_DIM_ELEMENT('X',-180,180,1),
  SDO_DIM_ELEMENT('Y',-90,90,1)
) , 4326);
COMMIT;

CREATE INDEX Ortsnetze_gebiet_spkt_ix
ON Ortsnetze (schwerpunkt(gebiet))
INDEXTYPE IS MDSYS.SPATIAL_INDEX;

SELECT * FROM Ortsnetze
WHERE SDO_FILTER (schwerpunkt(gebiet),SDO_GEOMETRY('POLYGON((8 53,8 54,9 54,9 53,8 53))',4326)) = 'TRUE';

-- ---------
-- FILTERUNG
-- ---------

--- Operator "SDO_FILTER"

-- R-Baum als Index:
ALTER INDEX GeoDbLand_geo_ix REBUILD;
-- Rechteckanfrage auf "GeoDbLand":
SELECT name
FROM GeoDbLand
WHERE SDO_FILTER (geo, SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

-- Quadtree als Index:
ALTER INDEX GeoDbLand_geo_ix REBUILD
PARAMETERS ('SDO_LEVEL=6');
-- Rechteckanfrage auf "GeoDbLand":
SELECT name
FROM GeoDbLand
WHERE SDO_FILTER (geo, SDO_GEOMETRY('POLYGON((5 5, 5 15, 15 15, 15 5, 5 5))')) = 'TRUE';

-- Rumliche Anfragen auf "Ortsnetze":

SELECT * FROM Ortsnetze
WHERE SDO_FILTER (gebiet, SDO_GEOMETRY('POLYGON((7.8 53.0,7.8 53.8,9.0 53.8,9.0 53.0,7.8 53.0))',4326)) = 'TRUE';

SELECT * FROM Ortsnetze
WHERE SDO_FILTER (gebiet, 
  SDO_CS.VIEWPORT_TRANSFORM( SDO_GEOMETRY(2003,0,null,SDO_ELEM_INFO_ARRAY(1,1003,3),
                                                      SDO_ORDINATE_ARRAY(7.8, 53.0, 9.0, 53.8) ), 4326)
) = 'TRUE';

SELECT n1.name
FROM Ortsnetze n1 INNER JOIN Ortsnetze n2
ON SDO_FILTER (n1.gebiet, n2.gebiet) = 'TRUE'
WHERE n2.name = 'Wildeshausen';

--- Operator "SDO_WITHIN_DISTANCE"

SELECT netz.name, SDO_GEOM.SDO_DISTANCE(netz.gebiet,gem.centrum,1e-5,'unit=km')
FROM Ortsnetze netz INNER JOIN Gemeinden gem
ON SDO_WITHIN_DISTANCE (netz.gebiet, gem.centrum,'distance=5 querytype=FILTER unit=km') = 'TRUE'
WHERE gem.name = 'Bad Zwischenahn';

